'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var coffee = _interopDefault(require('coffeescript'));
var path = _interopDefault(require('path'));
var rollupPluginutils = require('rollup-pluginutils');
var fs = _interopDefault(require('fs'));
var babylon = require('babylon');
var traverse = _interopDefault(require('@babel/traverse'));
var t = require('@babel/types');
var transform = _interopDefault(require('cfx.babel'));

var formatError = function (source, filename, err) {
  var carets, col, first_column, first_line, l, last_column, last_line, lines;
  ({
    first_line,
    last_line,
    first_column,
    last_column
  } = err.location);

  if (last_line == null) {
    // last_line no longer exists randomly
    last_line = first_line;
  } // get lines from source


  lines = source.split('\n'); // insert start of highlighting

  l = lines[first_line];
  lines[first_line] = l.substring(0, first_column) + '\x1B[91m' + l.substring(first_column); // insert end

  l = lines[last_line];

  if (first_line === last_line) {
    // error only spans one line and overlaps with current cursor
    // skip a bit farther due to insertion of highlighting
    col = last_column + 6;
  } else {
    col = last_column;
  } // end highlighting


  lines[last_line] = l.substring(0, col) + '\x1B[39m' + l.substring(col); // get subset of lines with error to display

  lines = lines.slice(first_line, last_line + 1); // underline with carets

  carets = Array(first_column + 1).join(' ') + `\x1B[91m${Array(last_column + 2 - first_column).join('^')}\x1B[39m`;
  lines.push(carets);
  return `${filename}:${first_line + 1}:${first_column + 1}: ${err.name}: ${err.message}\n${lines.join('\n')}\n`;
};

var sourceMap;
sourceMap = function (out) {
  if (out.v3SourceMap) {
    return JSON.parse(out.v3SourceMap);
  } else {
    return null;
  }
};

var coffee2 = function (opts = {}) {
  var filter;

  if (opts.bare == null) {
    opts.bare = true;
  }

  if (opts.sourceMap == null) {
    opts.sourceMap = true;
  }

  filter = rollupPluginutils.createFilter(opts.include, opts.exclude);
  return {
    name: 'coffee2',
    transform: function (code, id) {
      var err, out;

      if (!filter(id)) {
        return {
          code
        };
      }

      if (path.extname(id) !== '.coffee') {
        return {
          code
        };
      }

      try {
        out = coffee.compile(code, {
          filename: id,
          bare: opts.bare,
          sourceMap: opts.sourceMap
        });
      } catch (error) {
        err = error;

        if (err.location != null) {
          err.formattedMessage = formatError(code, id, err);
        }

        throw err;
      }

      return {
        code: out.js,
        map: sourceMap(out)
      };
    }
  };
};

// import dd from 'ddeyes'
var ASTToCode;
var getAST;
var rCFPath;
var ricffAST;
var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

rCFPath = filePath => {
  var fileObj, getFileObj;

  getFileObj = function (file) {
    return _extends({
      exists: fs.existsSync(file)
    }, path.parse(file));
  };

  fileObj = getFileObj(filePath);

  if (!fileObj.exists) {
    // file isnt exists
    fileObj = {
      coffee: function () {
        var _path;

        _path = path.format({
          root: path.join(fileObj.root, fileObj.dir + '/'),
          name: fileObj.base,
          ext: '.coffee'
        });
        return {
          path: _path,
          obj: getFileObj(_path)
        };
      }(),
      js: function () {
        var _path;

        _path = path.format({
          root: path.join(fileObj.root, fileObj.dir + '/'),
          name: fileObj.base,
          ext: '.js'
        });
        return {
          path: _path,
          obj: getFileObj(_path)
        };
      }()
    };

    if (fileObj.coffee.obj.exists) {
      return rCFPath(fileObj.coffee.path);
    }

    if (fileObj.js.obj.exists) {
      return rCFPath(fileObj.js.path);
    }

    return null; // file exists
  } else {
    if (fileObj.ext !== '') {
      // isnt dir
      return filePath;
    } else {
      return rCFPath(path.join(filePath, 'index.coffee'));
    }
  }
}; // replace import coffee file from AST


ricffAST = (ast, dirname = __dirname) => {
  traverse(ast, {
    ImportDeclaration: function (_ast) {
      var currentNode, fileParseObj, filePath;
      currentNode = _ast.node;
      fileParseObj = path.parse(currentNode.source.value);

      if (fileParseObj.root === '' && fileParseObj.dir === '') {
        return;
      }

      filePath = rCFPath(path.join(dirname, currentNode.source.value));

      if (filePath != null) {
        _ast.node.source = t.StringLiteral(filePath);
      }
    }
  });
  return ast;
};

getAST = source => {
  return babylon.parse(source, {
    sourceType: 'module'
  });
};

ASTToCode = function (ast) {
  return transform(ast);
};

var sourceMap$1;
sourceMap$1 = function (out) {
  if (out.v3SourceMap) {
    return JSON.parse(out.v3SourceMap);
  } else {
    return null;
  }
};

var coffeepath = function (opts = {}) {
  var filter;
  filter = rollupPluginutils.createFilter(opts.include, opts.exclude);
  return {
    name: 'coffeepath',
    transform: function (code, id) {
      if (!filter(id)) {
        return {
          code
        };
      }

      if (path.extname(id) !== '.coffee') {
        return {
          code
        };
      }

      return {
        // dd path.dirname id
        code: ASTToCode(ricffAST(getAST(code), path.dirname(id))),
        map: sourceMap$1(code)
      };
    }
  };
};

exports.coffee2 = coffee2;
exports.coffeepath = coffeepath;
